
Vn.Catalog = new Class
({
	Extends: Vn.Module

	,realmColor: null

	,activate: function ()
	{
		var model = this.get ('items');
		model.setTableInfo ('m', 'order_row_view');
		model.setFieldFlags ('id', Db.Conn.Flag.AI);	
		model.setFieldFlags ('Id_Article', Db.Conn.Flag.PRI_KEY);
	
		$('basket-button').addEventListener ('click', this.basketClicked.bind (this));
		$('catalog-menu-button').addEventListener ('click', this.showMenu.bind (this));
		$('catalog-menu').addEventListener ('click', this.onMenuClick.bind (this));
		$('search-entry').addEventListener ('change', this.onSearch.bind (this));

		this.get ('type-column').renderer = this.typeRenderer.bind (this);
		this.get ('cat').renderer = this.catRenderer;

		this.get ('realms-model').on ('status-changed', this.onRealmsReload.bind (this));
		this.get ('realms').on ('changed', this.onRealmChanged.bind (this));
		this.get ('types-model').on ('status-changed', this.onTypesReload.bind (this));
		this.get ('type').on ('changed', this.onTypeChanged.bind (this));
		this.get ('order-form').on ('status-changed', this.orderFormChanged.bind (this));
		this.get ('warehouse').value = 1;
		this.get ('date').value = new Date ();
		
		if (!Vn.Url.getQuery ('guest'))
		{
			var orderId = Vn.Cookie.getInt ('order');
		
			if (!orderId)
				this.configureView ();
			else
				this.get ('order-batch').addValue ('order', orderId);
		}
		else
			this.get ('order').value = 0;
	}

	,onRealmsReload: function (model, status)
	{
		if (status == Db.Model.Status.READY)
		{
			this.onRealmChanged ();
			this.onTypeChanged ();
		}
	}
	
	,onRealmChanged: function ()
	{
		var color;
		var realms = this.get ('realms-model');
		var row = realms.search ('id', this.get ('realm').value);

		if (row != -1)
			this.realmColor = '#'+ realms.get (row, 'color');
		else
			this.realmColor = null;
	}

	,onTypesReload: function (model, status)
	{
		if (status == Db.Model.Status.READY)
			this.refreshTitle ();
	}

	,onTypeChanged: function ()
	{
		if (Vn.isMobile ())
			this.hideMenu ();
			
		this.refreshTitle ();

		var color = null;
		
		if (this.get ('type').value)
			color = this.realmColor;

		$('catalog-header').style.backgroundColor = color;

		var itemsGrid = this.get ('items-grid').getNode ();
		var thead = itemsGrid.getElementsByTagName ('thead')[0];
		var tr = thead.getElementsByTagName ('tr')[0];
		tr.style.backgroundColor = color;
	}
	
	,refreshTitle: function (title)
	{
		var title = null;
		
		if (this.get ('type').value)
		{
			var types = this.get ('types-model');
			var row = types.search ('tipo_id', this.get ('type').value);

			if (row != -1)
				title = types.get (row, 'Tipo');
		}
		else if ($('search-entry').value)
			title = _('SearchResults');

		if (title)
			Vn.Node.setText ($('catalog-title'), title);
	}

	,onSearch: function (event)
	{
		var searchTags = $('search-entry').value;
		searchTags = searchTags != '' ? searchTags : null;

		var batch = this.get ('batch');
		batch.block ();

		this.get ('search').value = searchTags;

		if (searchTags)
		{
			this.get ('type').value = null;
			this.get ('realm').value = null;
		}

		batch.unblock ();
		batch.changed ();
	}

	,onMenuClick: function (event)
	{
		event.stopPropagation ();
	}

	,showMenu: function (event)
	{
		event.stopPropagation ();
		this.gui.showBackground ();
		$('catalog-menu').style.display = 'block';
		this.hideMenuCallback = this.hideMenu.bind (this);
		document.addEventListener ('click', this.hideMenuCallback);
	}

	,hideMenu: function ()
	{
		this.gui.hideBackground ();
		$('catalog-menu').style.display = 'none';
		document.removeEventListener ('click', this.hideMenuCallback);
		this.hideMenuCallback = null;
	}
	
	,configureView: function ()
	{
		var orderId = 0;
		var grid = this.get ('items-grid');
		var orderForm = this.get ('order-form');

		if (orderForm.numRows > 0)
		{
			orderForm.row = 0;
			orderId = orderForm.get ('id');

			Vn.Node.setText ($('basket-button'), _('ShoppingBasket'));

			var items = this.get ('items');
			items.updatable = true;
			items.setDefaultFromColumn ('item_id', 'm', 'Id_Article');
			items.setDefaultFromValue ('order_id', 'm', orderId);
			
			var amountCol = new Htk.ColumnSpin
			({
				 title: 'Amount'
				,editable: true
				,renderer: this.amountRender
			});
			amountCol.on ('changed', this.amountChanged.bind (this));
			grid.insertColumn (1, amountCol);
			
			var stemsCol = new Htk.ColumnSpin
			({
				 title: 'Stems'
				,editable: true
				,column: 'amount'
			});	
			grid.insertColumn (3, stemsCol);

			this.get ('warehouse').master = orderForm.getParam ('wh_id');
			this.get ('date').master = orderForm.getParam ('date_send');
		}
		else
			Vn.Cookie.unset ('order');

		this.get ('order').value = orderId;
	
		var priceCol = new Htk.ColumnSpin
		({
			 title: 'Price'
			,digits: 2
			,unit: '€'
			,column: 'price'
			,renderer: this.priceRenderer
		});
		grid.insertColumn (11, priceCol);
		
		$('basket-button').disabled = false;
	}
	
	,orderFormChanged: function (form)
	{
		if (form.ready)
			this.configureView ();
	}
	
	,basketClicked: function ()
	{
		this.hash.set ({'form': 'ecomerce/basket'});
	}
	
	,typeRenderer: function (column, form)
	{
		column.href = this.hash.make ({'type': form.get ('tipo_id')}, true);
	}
	
	,amountRender: function (renderer, form)
	{
		var amount = form.get ('amount');
		var grouping = form.get ('grouping');

		if (amount && grouping)
			renderer.value = amount / grouping;
		else
			renderer.value = null;
	}

	,amountChanged: function (renderer, row, newValue)
	{
		var model = this.get ('items');
		model.set (row, 'amount', newValue * model.get (row, 'grouping'));
	}

	,catRenderer: function (column, form)
	{
		switch (form.get ('Categoria'))
		{
			case 'A2':
				column.td.className = 'second-category';
				break;
			case 'B1':
				column.td.className = 'third-category';
				break;
			default:
				column.td.className = '';
		}
	}
	
	,priceRenderer: function (column, form)
	{
		if (form.get ('fixed'))
			column.td.className = 'fixed-price';
		else
			column.td.className = 'aprox-price';
	}
});

Htk.Realm = new Class
({
	Extends: Htk.Field
	,Tag: 'htk-realm'
	,Properties:
	{
		 model:
		 {
			type: Db.Model
			,set: function (x)
			{
				x.indexColumn ('id');
				this.link ({_model: x}, {'status-changed': this.onModelReload});
				this.onModelReload (x);
			}
			,get: function ()
			{
				return this._model;
			}
		}
	}

	,_model: null
	,selectedImg: null

	,initialize: function (mod)
	{
		this.createElement ('div');
		this.node.className = 'htk-realm';
	}

	,onModelReload: function (model)
	{
		Vn.Node.removeChilds (this.node);
	
		if (model.status != Db.Model.Status.READY)
			return;

		var width = parseInt (100 / model.numRows) +'%';

		for (var i = 0; i < model.numRows; i++)
		{
			var id = model.get (i, 'id');
		
			var img = document.createElement ('img');
//			img.src = Vn.Config['image_dir'] +'/family/full/'+ id +'.png';
			img.src = 'image/family/'+ id +'.svg';
			img.title = model.get (i, 'reino');
			img.alt = img.title;
			img.style.width = width;
			img.style.backgroundColor = '#'+ model.get ('color');
			img.addEventListener ('click', this.clicked.bind (this, img, id));
			this.node.appendChild (img);
			
			if (id == this.value)
				this.focusImage (img);
		}
	}
	
	,focusImage: function (img)
	{
		if (this.selectedImg)
			this.selectedImg.className = '';

		img.className = 'selected';
		this.selectedImg = img;
	}

	,clicked: function (img, value)
	{
		this.focusImage (img);
	
		if (value != this.value)
			this.valueChanged (value);
	}
	
	,putValue: function (value)
	{
		var row = this._model.search ('id', value);

		if (row != -1)
			this.focusImage (this.node.childNodes[row]);
	}
});

