
Vn.Orders = new Class
({
	Extends: Vn.Module

	,activate: function ()
	{
		Vn.get ('pay-button').addEventListener ('click', this.onPayButtonClick.bind (this));
		Vn.get ('start-order').addEventListener ('click', this.onStartClick.bind (this));
		this.get ('company-pay').on ('clicked', this.onCompanyPayClick, this);
		this.get ('edit-order').on ('clicked', this.onContinueClick, this);
		this.get ('edit-ticket').on ('clicked', this.onShowClick, this);
		this.get ('ticket-pay').on ('clicked', this.onTicketPayClick, this);
		this.get ('ticket-pay').renderer = this.payRenderer;
		this.get ('debt-amount').conditionalFunc = this.debtConditionalFunc;

		this.payPopup = new Vn.NodePopup ({htmlNode: Vn.get ('debt-popup')});
		
		// Ends the transaction
		
		var tpvStatus = this.hash.get ('tpv_status');
		
		if (tpvStatus)
		{
			var batch = new Sql.Batch ();
			batch.addValue ('transaction', this.hash.get ('tpv_order'));
			batch.addValue ('status', tpvStatus);
		
			var query = 'CALL transaction_end (#transaction, #status)';
		
			this.conn.execQuery (query, 
				this.onTransactionEnd.bind (this), batch);
		}
	}

	,onStartClick: function ()
	{
		Vn.Cookie.unset ('order');
		this.hash.set ({'form': 'ecomerce/basket'});
	}

	,onContinueClick: function (column, orderId)
	{
		this.hash.set ({
			'form': 'ecomerce/basket',
			'order': orderId
		});
	}

	,onShowClick: function  (column, ticketId)
	{
		this.hash.set ({
			'form': 'ecomerce/ticket',
			'ticket': ticketId
		});
	}
	
	,payRenderer: function (column, form)
	{
		if (form.get ('company_id') && form.get('total'))
			column.td.className = '';
		else
			column.td.className = 'hide';
	}
	
	// TPV
	
	,debtConditionalFunc: function (field, value)
	{
		if (value > 0)
			field.node.className = 'positive-debt';
		else
			field.node.className = 'negative-debt';
	}
	
	,onPayButtonClick: function ()
	{
		this.payPopup.showPopup (Vn.get ('pay-button'));
	}
	
	,onCompanyPayClick: function (column, value, row)
	{
		var model = this.get ('balance');
		var company = model.get (row, 'id');
		var amount = model.get (row, 'amount');

		amount = amount <= 0 ? null : amount;

		this.pay (amount, 40000, company);
	}

	,onTicketPayClick: function (column, value, row)
	{
		var model = this.get ('tickets');
		var company = model.get (row, 'company_id');
		var total = model.get (row, 'total');
		this.pay (total, total, company);
	}

	,pay: function (defaultAmount, maxAmount, company)
	{
		var defaultAmountStr = '';
		
		if (defaultAmount !== null)
			defaultAmountStr = Vn.Value.format (defaultAmount, '%.2d');

		var amount = parseFloat (prompt (_('AmountToPay:'), defaultAmountStr));
	
		if (amount > 0 && (maxAmount === null || amount <= maxAmount))
		{
			var query = 'CALL transaction_start (#company, #amount)';

			var batch = new Sql.Batch ();
			batch.addValue ('company', company);
			batch.addValue ('amount', parseInt (amount * 100));
			
			this.conn.execQuery (query,
				this.onTransactionStart.bind (this), batch);
		}
		else if (!isNaN (amount))
			alert (_('AmountError'))
	}

	,getTpvUrl: function (status, order)
	{
		var path = location.protocol +'//'+ location.host;
		path += location.port ? ':'+ location.port : '';
		path += location.pathname;
		path += location.search ? location.search : '';
		path += this.hash.make ({
			'tpv_status': status,
			'tpv_order': order
		}, true);

		return path;
	}

	,onTransactionStart: function (resultSet)
	{
		var res = resultSet.fetchResult ();

		if (res && res.next ())
		{
			var transactionId = res.get ('ds_order');

			var form = Vn.get ('tpv-form');
			var formMap =
			{
				 'Ds_Merchant_Amount': 'amount'
				,'Ds_Merchant_Order': 'ds_order'
				,'Ds_Merchant_MerchantCode': 'id'
				,'Ds_Merchant_Currency': 'currency'
				,'Ds_Merchant_TransactionType': 'transaction_type'
				,'Ds_Merchant_Terminal': 'terminal'
				,'Ds_Merchant_MerchantURL': 'merchant_url'
				,'Ds_Merchant_MerchantSignature': 'signature'
			};

			form.action = res.get ('url');

			for (var name in formMap)
				form[name].value = res.get (formMap[name]);

			form['Ds_Merchant_UrlOK'].value = this.getTpvUrl ('ok', transactionId);
			form['Ds_Merchant_UrlKO'].value = this.getTpvUrl ('ko', transactionId);
			
			// Showing TPV on the same window
			
			form.submit ();
		}
		else
			alert (_('PayError'));
	}
	
	,onTransactionEnd: function ()
	{
		this.get ('tickets').refresh ();
	}
});

Vn.NodePopup = new Class
({
	Extends: Htk.Widget
	,Implements: Htk.Popup
	,Properties:
	{
		htmlNode:
		{
			type: Object
			,set: function (x)
			{
				if (x.parentNode)
					Vn.Node.remove (x);
				
				x.style.display = 'block';					
				this.node = x;
			}
			,get: function (x)
			{
				return this.node;
			}
		}
	}

	,initialize: function (props)
	{
		this.parent (props);
	}
});

